<?php

// No direct access
if (!defined('ABSPATH')) {
    die('Error #845173685');
}

class AddPipe
{

    /**
     * Account hash holder
     * @var string
     */
    public $accountHash = '';

    /**
     * Recording environment ID holder
     * @var string
     */
    public $envId = '';

    /**
     * Recording width holder
     * @var string
     */
    public $widthVal = '600px';

    /**
     * Recording height holder
     * @var int
     */
    public $heightVal = 390;

    /**
     * Recording length holder
     * @var int
     */
    public $recordingLength = 20;

    /**
     * Recording quality holder
     * @var string
     */
    public $recordingQuality = '480p';

    /**
     * Auto Save Recording holder
     * @var string
     */
    public $asv = '1';

    /**
     * Audio Only holder
     * @var string
     */
    public $ao = '0';

    /**
     * Show Menu holder
     * @var string
     */
    public $showMenu = '1';

    /**
     * Skip Initial Screen holder
     * @var string
     */
    public $sis = '0';

    /**
     * Desktop Upload holder
     * @var string
     */
    public $dup = '0';

    /**
     * Screen + Camera rec holder
     * @var string
     */
    public $srec = '0';

    /**
     * Countdown timer holder
     * @var string
     */
    public $countdowntimer = '0';

    /**
     * Background blurring holder
     * @var string
     */
    public $bgblur = '0';

    /**
     * Plugin instance
     * @var
     */
    private static $_instance;

    /**
     * Plugin screen name
     * @var string
     */
    public $_pluginScreenName = 'AddPipe';

    /**
     * L'Constructeur
     */
    public function __construct()
    {
        // Get the user's account hash from the database
        $this->accountHash = $this->getAccountHash();

        // Retrieve the environment code from the database
        $this->envId = $this->getEnvId();
    }

    /**
     * Hook our methods into WordPress
     */
    public function run()
    {
        /* Draw the menu */
        add_action('admin_menu', array($this, 'drawMenu'));

        /* AJAX: save settings */
        add_action('wp_ajax_addpipe_ajax_save_settings', array($this, 'addpipe_ajax_save_settings'));
        /* AJAX: Generate a shortcode for the recording client */
        add_action('wp_ajax_addpipe_ajax_shortcode_generator', array($this, 'addpipe_ajax_shortcode_generator'));
        /* Hook the function to handle the AJAX request */
        add_action('wp_ajax_delete_recording', array($this, 'addpipe_ajax_delete_recording'));
        /* AJAX: sync deleted recordings */
        add_action('wp_ajax_addpipe_ajax_sync_deleted', array($this, 'addpipe_ajax_sync_deleted'));

        /* Listen to incoming hooks from addpipe.com */
        add_action('wp_ajax_nopriv_addpipeWebhook', array($this, 'addpipeWebhook'));
        /* The [pipe_recorder] shortcode */
        add_shortcode('pipe_recorder', array($this, 'newRecorder'));
        /* The [pipe_playback] */
        add_shortcode('pipe_playback', array($this, 'newPlayback'));

        // Enqueue scripts (css and js) in the admin area - only on our plugin pages
        add_action('admin_enqueue_scripts', array($this, 'enqueueBackendScripts'));
        // Enqueue scripts (css and js) in the frontend area
        add_action('wp_enqueue_scripts', array($this, 'enqueueFrontendScripts'));
    }

    public function enqueueBackendScripts()
    {
        wp_enqueue_script('pipejs', 'https://cdn.addpipe.com/2.0/pipe.min.js', array(), '2.0', false);
        wp_enqueue_style('pipecss', 'https://cdn.addpipe.com/2.0/pipe.css', array(), '2.0');
    }

    public function enqueueFrontendScripts()
    {
        wp_enqueue_script('pipejs', 'https://cdn.addpipe.com/2.0/pipe.min.js', array(), '2.0', false);
        wp_enqueue_style('pipecss', 'https://cdn.addpipe.com/2.0/pipe.css', array(), '2.0');
    }

    // Function to initialize WP_Filesystem
    public function initialize_wp_filesystem()
    {
        if (!function_exists('WP_Filesystem')) {
            require_once(ABSPATH . 'wp-admin/includes/file.php');
        }

        WP_Filesystem();
    }


    /**
     * Install method
     */
    static function install()
    {
        // Include the necessary file for dbDelta function
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // General settings
        update_option('AddPipe' . 'AccountHash', '');
        update_option('AddPipe' . 'EnvId', '');

        // Install the custom tables
        global $wpdb;

        // Define the table schema
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "
                      CREATE TABLE IF NOT EXISTS
                      `{$wpdb->prefix}addpipe_records`
                        (
                        `internal_id` int unsigned NOT NULL AUTO_INCREMENT,
                        `user_id` bigint unsigned DEFAULT NULL,
                        `environment_code` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `stream_name` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_id` int NOT NULL,
                        `recording_length` decimal(9,2) DEFAULT NULL,
                        `recording_size` int DEFAULT NULL,
                        `recording_url` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_raw_url` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_img_url` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_width` SMALLINT(255) unsigned DEFAULT NULL,
                        `recording_height` SMALLINT(255) unsigned DEFAULT NULL,
                        `post_id` bigint unsigned DEFAULT '0',
                        `extension` VARCHAR(25),
                        `date` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
                        `json` varchar(3000) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_520_ci NOT NULL,
                        PRIMARY KEY (`internal_id`)
                        ) $charset_collate;
                        ";

        // Execute the query using dbDelta
        dbDelta($sql);

        // Remove "recording_"
        $sql = "
                      CREATE TABLE IF NOT EXISTS
                      `{$wpdb->prefix}addpipe_shortcodes`
                        (
                        `shortcode_id` INT(6) NOT NULL AUTO_INCREMENT,
                        `length` INT(11),
                        `quality` VARCHAR(5),
                        `env_code` CHAR(6),
                        `width` VARCHAR(25), 
                        `height` INT(20),
                        `asv` SMALLINT(1) DEFAULT 1,
                        `ao` SMALLINT(1) DEFAULT 0,
                        `showMenu` SMALLINT(1) DEFAULT 1,
                        `sis` SMALLINT(1) DEFAULT 0,
                        `dup` SMALLINT(1) DEFAULT 0,
                        `srec` SMALLINT(1) DEFAULT 0,
                        `countdowntimer` INT(11) DEFAULT 0,
                        `bgblur` SMALLINT(1) DEFAULT 0,
                        `post_id` INT(11) DEFAULT 0,
                        `date` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                        PRIMARY KEY (`shortcode_id`)
                        ) $charset_collate;
        ";

        // Execute the query using dbDelta
        dbDelta($sql);


        //Add the pipe_access_plugin capability to administrator role
        $role = get_role('administrator');

        //add the umbrella capability
        $role->add_cap('pipe_access_plugin', true);

        //add individual capabilities for each fo the 4 menus of the plugin
        $role->add_cap('pipe_access_record', true);
        $role->add_cap('pipe_access_embed', true);
        $role->add_cap('pipe_access_recordings', true);
        $role->add_cap('pipe_access_setup', true);


        // Log any error to file
        // file_put_contents(dirname(__FILE__) . '/_plugin_error_activation.log', ob_get_contents(), FILE_APPEND);
        if (!function_exists('WP_Filesystem')) {
            require_once(ABSPATH . 'wp-admin/includes/file.php');
        }

        WP_Filesystem();
        global $wp_filesystem;
        // Log any error to file
        $log_file_path = dirname(__FILE__) . '/_plugin_error_activation.log';
        $log_content = ob_get_contents();

        if ($wp_filesystem) {
            $wp_filesystem->put_contents($log_file_path, $log_content, FILE_APPEND);
        }
    }

    /**
     * Uninstall plugin method
     */
    static function uninstall()
    {

        // Include the necessary file for dbDelta function
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // Delete the general settings
        delete_option('AddPipe' . 'AccountHash');
        delete_option('AddPipe' . 'EnvId');

        //remove the capabilities added to the administrator
        $role = get_role('administrator');
        $role->remove_cap('pipe_access_plugin');
        $role->remove_cap('pipe_access_record');
        $role->remove_cap('pipe_access_embed');
        $role->remove_cap('pipe_access_recordings');
        $role->remove_cap('pipe_access_setup');

        // Delete custom tables
        global $wpdb;
        $sql = "DROP TABLE `{$wpdb->prefix}addpipe_records`";

        // Execute the query using dbDelta
        dbDelta($sql);

        $sql = "DROP TABLE `{$wpdb->prefix}addpipe_shortcodes`";

        // Execute the query using dbDelta
        dbDelta($sql);
    }

    /**
     * Returns the post/page ID, whether it's inside The Loop (or outside)
     * @return false|int
     */
    public function getThePostId()
    {

        global $wpdb, $post, $wp_query;

        if (in_the_loop()) {
            return get_the_ID();
        } else {
            return $wp_query->get_queried_object_id();
        }

    }

    /**
     * Returns the AddPipe instance
     * @return AddPipe
     */
    static function getInstance()
    {
        if (!isset(self::$_instance)) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Class name
     * @return string
     */
    public function __toString()
    {
        return 'AddPipe';
    }

    /**
     * Returns the plugin folder path
     * @return string
     */
    public function getPluginFolder()
    {
        return dirname(dirname(__FILE__));
    }

    /**
     * Returns the plugin web address (URL)
     * @return string
     */
    public function getPluginUrl()
    {
        return plugins_url() . '/pipe-recorder/';
    }

    /**
     * Returns the plugin's server path, eg /srv/users/serverpilot/apps/dev/public/wordpress/wp-content/plugins/pipe-recorder/
     * @return string
     */
    public function getPluginPath()
    {
        return plugin_dir_path(__FILE__);
    }

    /**
     * Return the URL to the static folder (css, js, img)
     * @param $staticFileOrFolder
     * @return string
     */
    public function getPluginViewURL($staticFileOrFolder)
    {
        return $this->getPluginUrl() . 'static/' . $staticFileOrFolder;
    }

    /**
     * Returns the path to the Views folder
     * @return string
     */
    public function getViewsPath()
    {
        return $this->getPluginPath() . 'views/';
    }

    /**
     * Loads the appropriate controller based on URL
     * @return RecordedRecordings|RecordNew|Settings
     * @throws Exception
     */
    public function loadController()
    {
        if (isset($_GET) && !empty($_GET['page']) && !(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'loadController'))) {
            $page = $_GET['page'];
            switch ($page):
                case 'pipe-recorder/load/settings':
                    require_once $this->getPluginPath() . 'Settings.php';
                    return new Settings();
                    break;
                case 'pipe-recorder/load/recordings':
                    require_once $this->getPluginPath() . 'RecordedRecordings.php';
                    return new RecordedRecordings();
                    break;
                case 'pipe-recorder':
                    require_once $this->getPluginPath() . 'RecordNew.php';
                    return new RecordNew();
                    break;
                case 'pipe-recorder/load/embed-recorder':
                    require_once $this->getPluginPath() . 'EmbedRecorder.php';
                    return new EmbedRecorder();
                    break;
                default:
                    include_once $this->getPluginPath() . '../views/not_found.php';
                    throw new Exception('Not found. #98645312');
            endswitch;
        }
    }

    /**
     * "Draws" the plugin menu
     * @return string
     */
    public function drawMenu()
    {
        if (current_user_can('pipe_access_plugin')) {
            $this->_pluginScreenName = add_menu_page(
                // $page_title - The text to be displayed in the title tags of the page when the menu is selected
                'Pipe Recorder',
                // $menu_title - The on-screen name text for the menu
                'Pipe Recorder',
                // $capability - The capability required for this menu to be displayed to the user
                'pipe_access_plugin',
                // $menu_slug - The slug name to refer to this menu by (should be unique for this menu). Prior to Version 3.0 this was called the file (or handle) parameter.
                // If the function parameter is omitted, the menu_slug should be the PHP file that handles the display of the menu page content.
                'pipe-recorder',
                // function - (optional) The function that displays the page content for the menu page.
                // Default: None. Technically, the function parameter is optional, but if it is not supplied, then WordPress will assume that including the PHP file will generate the administration screen,
                // without calling a function. Most plugin authors choose to put the page-generating code in a function within their main plugin file. In the event that the function parameter is specified,
                // it is possible to use any string for the menu_slug parameter. This allows usage of pages such as ?page=my_super_plugin_page instead of ?page=my-super-plugin/admin-options.php.
                array($this, 'loadController'),
                // $icon_url
                plugins_url('/pipe-recorder/static/img/logo.png'),
                // position
                81
            );

            // add_submenu_page($parent_slug, $page_title, $menu_title, $capability, $menu_slug, $function);
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Create Recording', 'pipe_access_record', 'pipe-recorder', array($this, 'loadController'));
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Embed Recorder', 'pipe_access_embed', 'pipe-recorder/load/embed-recorder', array($this, 'loadController'));
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Recordings', 'pipe_access_recordings', 'pipe-recorder/load/recordings', array($this, 'loadController'));
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Settings', 'pipe_access_setup', 'pipe-recorder/load/settings', array($this, 'loadController'));

            // If there is no account hash, probably the user doesn't have an addpipe.com account
//             if (empty($this->accountHash)) {
//                 add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Get Trial', 'manage_options', 'pipe-recorder/load/get-trial', array($this, 'loadController'));
//             }
        }

        // Log any error to file
        // file_put_contents(dirname(__FILE__) . '/_plugin_error_menu_draw.log', ob_get_contents(), FILE_APPEND);
        $this->initialize_wp_filesystem();
        global $wp_filesystem;
        // Log any error to file
        $log_file_path = dirname(__FILE__) . '/_plugin_error_menu_draw.log';
        $log_content = ob_get_contents();

        if ($wp_filesystem) {
            $wp_filesystem->put_contents($log_file_path, $log_content, FILE_APPEND);
        }
    }

    /**
     * Listen to any webhooks from addpipe.com
     */
    public function addpipeWebhook()
    {
        // Check if the request has the correct content type and payload
        if ($_SERVER['CONTENT_TYPE'] === 'application/json' && !(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'addpipeWebhook'))) {

            $this->initialize_wp_filesystem();
            global $wp_filesystem;
            global $wpdb;

            // Get the raw POST data
            $json = file_get_contents('php://input');
            $json = json_decode($json);

            if ($json && ($json->event == "video_recorded" || $json->event == "video_converted" || $json->event == "video_copied_pipe_s3")) {
                /* Slice the custom values from the payload */
                $payload = explode(',', $json->data->payload);
                $accountHash = filter_var($payload[0], FILTER_SANITIZE_STRING);
                $userId = filter_var($payload[1], FILTER_SANITIZE_NUMBER_INT);
                $postId = filter_var($payload[2], FILTER_SANITIZE_NUMBER_INT);
                // $envCode = filter_var($payload[3], FILTER_SANITIZE_STRING);

                /* Build the Insert query */
                $jsonData = json_encode($json); // Encode the Object back to json

                /* Check the contents and the account hash */
                if ($this->accountHash == $accountHash) {
                    switch ($json->event) {
                        case "video_recorded":
                            $sql =
                                $wpdb->prepare(
                                    "
                                INSERT INTO `{$wpdb->prefix}addpipe_records`
                                (
                                    `user_id`,
                                    `recording_id`,
                                    `post_id`,
                                    `environment_code`,
                                    `stream_name`,
                                    `extension`,
                                    `json`
                                )
                                VALUES
                                (
                                    %d, /* user_id from Payload, should be an integer */
                                    %d, /* recording_id, should be an integer */
                                    %d, /* post_id, should be an integer */
                                    %s, /* environment_code from Payload, should be a string */
                                    %s, /* stream_name, should be a string */
                                    %s, /* extension, should be a string */
                                    %s  /* jsonData, should be a string */
                                );
                                ",
                                    array(
                                        $userId,
                                        $json->data->id,
                                        $postId,
                                        $json->data->envCode,
                                        $json->data->videoName,
                                        $json->data->type,
                                        $jsonData
                                    )
                                )
                            ;

                            // Execute the query
                            $wpdb->query($sql);
                            break;
                        case "video_converted":
                            $sql =
                                $wpdb->prepare(
                                    "
                                UPDATE `{$wpdb->prefix}addpipe_records` SET
                                `recording_size` = %d,
                                `recording_length` = %s,
                                `recording_width` = %d,
                                `recording_height` = %d
                                WHERE `recording_id` = %d;
                                ",
                                    array(
                                        $json->data->size,
                                        $json->data->duration,
                                        $json->data->width,
                                        $json->data->height,
                                        $json->data->id
                                    )
                                )
                            ;

                            // Execute the query
                            $wpdb->query($sql);
                            break;
                        case "video_copied_pipe_s3":
                            $sql =
                                $wpdb->prepare(
                                    "
                                UPDATE `{$wpdb->prefix}addpipe_records` SET
                                `recording_url` = %s, 
                                `recording_img_url` = %s,
                                `recording_raw_url` = %s,
                                `recording_size` = %d
                                WHERE `recording_id` = %d;
                                ",
                                    array(
                                        $json->data->url,
                                        $json->data->snapshotUrl,
                                        $json->data->rawRecordingUrl,
                                        $json->data->size,
                                        $json->data->id
                                    )
                                )
                            ;

                            // Execute the query
                            $wpdb->query($sql);
                            break;
                        default:
                            // Unsupported type or empty JSON
                            http_response_code(400);
                            die(esc_attr("400 - Bad Request: Unsupported type or empty payload"));
                    }

                    // insert/update db ; give something back and die
                    http_response_code(200);
                    die('200 - OK.');
                } else {

                    // The Payload isn't correct
                    http_response_code(417);
                    $_POST['payload'] .= "------------- Error 417 -------------" . PHP_EOL;
                    // file_put_contents(dirname(__FILE__) . '/_addpipe_errors.log', print_r($_POST['payload'], true) . PHP_EOL, LOCK_EX | FILE_APPEND);
                    $log_file_path = dirname(__FILE__) . '/_addpipe_errors.log';
                    $log_content = print_r($_POST['payload'], true);

                    if ($wp_filesystem) {
                        $wp_filesystem->put_contents($log_file_path, $log_content, PHP_EOL, LOCK_EX | FILE_APPEND);
                    }
                    die('417 - Expectation Failed. The Payload is not correct.');

                }

                // Default: $_POST['payload'] exists, but could not be proccessed
                http_response_code(501);
                $_POST['payload'] .= "------------- Unknown error -------------" . PHP_EOL;
                // file_put_contents(dirname(__FILE__) . '/_addpipe_errors.log', print_r($_POST['payload'], true) . PHP_EOL, LOCK_EX | FILE_APPEND);
                $log_file_path = dirname(__FILE__) . '/_addpipe_errors.log';
                $log_content = print_r($_POST['payload'], true);

                if ($wp_filesystem) {
                    $wp_filesystem->put_contents($log_file_path, $log_content, PHP_EOL, LOCK_EX | FILE_APPEND);
                }
                die('501 - Not implemented. I cannot process the payload.');
            } else {
                http_response_code(400);
                die(esc_attr("400 - Bad Request: Unsupported event type: '{$json->event}'"));
            }
        } else {
            // Unsupported type or empty JSON
            http_response_code(400);
            die(esc_attr("400 - Bad Request: Unsupported type or empty payload"));
        }
    }

    /**
     * Translates a given SIZE in a human readable format
     * @param $bytes
     * @param int $decimals
     * @return string
     */
    public function filesizeForHumans($bytes, $decimals = 2)
    {
        $size = array(' B', ' KB', ' MB', ' GB', ' TB', ' PB', ' EB', ' ZB', ' YB');
        $factor = floor((strlen($bytes) - 1) / 3);

        return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . @$size[$factor];
    }

    /**
     * Selects from DB only active recordings
     * @param int $limit
     * @param int $offset
     * @return array|null|object
     */
    public function getRecordedRecordings($limit = 25, $offset = 0)
    {

        global $wpdb;

        $limit = filter_var($limit, FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($offset, FILTER_SANITIZE_NUMBER_INT);

        // Create a unique cache key based on the function arguments
        $cache_key = 'recorded_recordings_' . $limit . '_' . $offset;
        $cache_group = 'recorded_recordings_group';

        // Attempt to get the results from cache
        $results = wp_cache_get($cache_key, $cache_group);

        if ($results !== false) {
            return $results;
        }

        // return only 25 records, start on record 0 (OFFSET 0)
        // phpcs:disable
        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                                SELECT * FROM {$wpdb->prefix}addpipe_records
                                ORDER BY `date` DESC
                                LIMIT %d OFFSET %d
                                ",
                array(
                    $limit,
                    $offset
                )
            )
        );
        // phpcs:enable

        // Store the results in cache
        wp_cache_set($cache_key, $results, $cache_group, 30); // Cache for 1 minute

        return $results;
    }

    /**
     * Selects and returns from DB active recorder shortcodes
     * @param int $limit
     * @param int $offset
     * @return array|null|object
     */
    public function getGeneratedShortcodes($limit = 25, $offset = 0)
    {

        global $wpdb;

        $limit = filter_var($limit, FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($offset, FILTER_SANITIZE_NUMBER_INT);

        // Create a unique cache key based on the function arguments
        $cache_key = 'addpipe_shortcodes_' . $limit . '_' . $offset;
        $cache_group = 'addpipe_shortcodes_group';

        // Attempt to get the results from cache
        $results = wp_cache_get($cache_key, $cache_group);

        if ($results !== false) {
            return $results;
        }

        // return only 25 records, start on record 0 (OFFSET 0)
        // phpcs:disable
        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                                SELECT * FROM {$wpdb->prefix}addpipe_shortcodes
                                ORDER BY `date` DESC
                                LIMIT %d OFFSET %d
                                ",
                array(
                    $limit,
                    $offset
                )
            )
        );
        // phpcs:enable

        // Store the results in cache
        wp_cache_set($cache_key, $results, $cache_group, 30); // Cache for 1 minute

        return $results;
    }

    /**
     * Check remote URL if exists (used to Sync Deleted Recordings)
     * @param $url
     * @return bool
     */
    public function isFileOnServer($url)
    {
        if (empty($url))
            return true; // File has not been converted yet
        $serverResponse = wp_remote_get(filter_var($url, FILTER_SANITIZE_URL));
        $serverResponse = $serverResponse['response']['code'];

        if ($serverResponse === 403 || $serverResponse === 404) {
            return false;
        }

        return true;

    }

    /* ------------------------------------------------------------------------------------------
     * AJAX CALLS
     * ------------------------------------------------------------------------------------------
     */

    /**
     * AJAX: generate a shortcode for the recorder - EmbedRecorder.php
     */
    public function addpipe_ajax_shortcode_generator()
    {
        if (isset($_POST['action']) && $_POST['action'] === 'addpipe_ajax_shortcode_generator' && !(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'addpipe_ajax_shortcode_generator'))) {

            $length = filter_var($_POST['length'], FILTER_SANITIZE_NUMBER_INT);
            $quality = filter_var($_POST['quality'], FILTER_SANITIZE_STRING);
            $env = filter_var($_POST['env'], FILTER_SANITIZE_STRING);
            $width = filter_var($_POST['width'], FILTER_SANITIZE_STRING);
            $height = filter_var($_POST['height'], FILTER_SANITIZE_NUMBER_INT);
            $asv = filter_var($_POST['asv'], FILTER_SANITIZE_NUMBER_INT);
            $ao = filter_var($_POST['ao'], FILTER_SANITIZE_NUMBER_INT);
            $showMenu = filter_var($_POST['showMenu'], FILTER_SANITIZE_NUMBER_INT);
            $sis = filter_var($_POST['sis'], FILTER_SANITIZE_NUMBER_INT);
            $dup = filter_var($_POST['dup'], FILTER_SANITIZE_NUMBER_INT);
            $srec = filter_var($_POST['srec'], FILTER_SANITIZE_NUMBER_INT);
            $countdowntimer = filter_var($_POST['countdowntimer'], FILTER_SANITIZE_NUMBER_INT);
            $bgblur = filter_var($_POST['bgblur'], FILTER_SANITIZE_NUMBER_INT);

            // Define data to be inserted into DB
            $data = array(
                'length' => $length,
                'quality' => $quality,
                'env_code' => $env,
                'width' => $width,
                'height' => $height,
                'asv' => $asv,
                'ao' => $ao,
                'showMenu' => $showMenu,
                'sis' => $sis,
                'dup' => $dup,
                'srec' => $srec,
                'countdowntimer' => $countdowntimer,
                'bgblur' => $bgblur,
            );

            // Define data format
            $data_format = array('%d', '%s', '%s', '%s', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d');

            global $wpdb;
            // Perform the insert operation using prepared statement
            // phpcs:disable
            $result = $wpdb->insert("{$wpdb->prefix}addpipe_shortcodes", $data, $data_format);
            // phpcs:enable

            if ($result === false) {
                // Error occurred while inserting into the database
                $error_message = esc_attr($wpdb->last_error);
                echo wp_kses_post("<h3 style='color: red;'>Error occurred: $error_message</h3>");
            } else {
                // phpcs:disable
                $shortcode = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}addpipe_shortcodes` ORDER BY `date` DESC LIMIT 1");
                // phpcs:enable
                $shortcode = $shortcode[0];
                $id = esc_attr($shortcode->shortcode_id);

                echo wp_kses_post("
                    <h3 style='color: darkgreen;'>Shortcode created </h3>
                    <h4>Now you can use the shortcode <code>[pipe_recorder {$id}]</code> in any page or post.</h4>
                ");
            }

            // Kill the script, according to WP
            wp_die();
        }
    }

    /**
     * AJAX: save data from the Settings.php
     */
    public function addpipe_ajax_save_settings()
    {
        if (isset($_POST['action']) && $_POST['action'] === 'addpipe_ajax_save_settings' && !(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'addpipe_ajax_save_settings'))) {

            if (isset($_POST['accountHash'])) {
                $this->setAccountHash($_POST['accountHash']);
            }

            if (isset($_POST['envId'])) {
                $this->setEnvId($_POST['envId']);
            }

            echo "
                <h4 style='color: green;'>Settings have been updated </h4>
                <p class='update-nag'>
                    Your recordings will not be visible in the Recordings menu until you register the correct webhook. <a href='https://addpipe.com/docs/plugins/wordpress-plugin/' target='_blank'>Learn how</a>.
                </p>
            ";

            // Kill the script, according to WP
            wp_die();
        }
    }

    /**
     * AJAX: delete the recording - RecordedRecordings.php
     */
    public function addpipe_ajax_delete_recording()
    {
        // Check if the request is valid
        if (isset($_POST['action']) && $_POST['action'] === 'delete_recording' && wp_verify_nonce($_POST['_wpnonce'], 'delete_recording_nonce')) {
            global $wpdb;

            // Get the recording ID from the POST request
            $recording_id = intval($_POST['recording_id']);

            // Prepare the SQL query to delete the recording from the database
            $table_name = $wpdb->prefix . 'addpipe_records';
            $sql = $wpdb->prepare("DELETE FROM $table_name WHERE recording_id = %d", $recording_id);

            // Execute the query
            $deleted = $wpdb->query($sql);

            if ($deleted !== false) {
                // Send a success response
                wp_send_json_success(array('message' => 'Recording deleted successfully.'));
            } else {
                // Send a failure response
                wp_send_json_error(array('message' => 'Failed to delete recording.'));
            }

            wp_die(); // Terminate the script
        } else {
            // Send an error response if the request is not valid
            wp_send_json_error(array('message' => 'Invalid request.'));
            wp_die(); // Terminate the script
        }
    }

    public function addpipe_ajax_sync_deleted()
    {
        if (isset($_POST['action']) && $_POST['action'] === 'addpipe_ajax_sync_deleted' && wp_verify_nonce($_POST['_wpnonce'], $_POST['action'])) {

            // Include the necessary file for dbDelta function
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            global $wpdb;

            foreach ($this->getRecordedRecordings() as $arr => $obj) {

                // Check the URL; if it returns 403 or 404, delete the entry from the database
                if ($this->isFileOnServer($obj->recording_url)) {
                    echo esc_attr("Recording {$obj->recording_id} OK | ");
                } else {
                    echo esc_attr("Recording {$obj->recording_id} Deleted | ");
                    $sql = $wpdb->prepare("UPDATE {$wpdb->prefix}addpipe_records SET `active` = 0 WHERE `recording_url` = %s LIMIT 1", $obj->recording_url);
                    // Execute the query using dbDelta
                    dbDelta($sql);
                }

            }

            echo "<strong> Sync completed </strong>";

            // Kill the script, according to WP
            wp_die();

        } else {

            echo "Request failed. Error #846531";
            wp_die();
        }
    }

    /* ------------------------------------------------------------------------------------------
     * GETTERS AND SETTERS
     * ------------------------------------------------------------------------------------------
     */

    /**
     * Get the account hash
     * @return string
     */
    public function getAccountHash()
    {
        $this->accountHash = get_option($this->_pluginScreenName . 'AccountHash');
        return (string) $this->accountHash;
    }

    /**
     * Get the environment code
     * @return string
     */
    public function getEnvId()
    {
        $this->envId = get_option($this->_pluginScreenName . 'EnvId');
        return (string) $this->envId;
    }

    /**
     * Set the account hash
     * @param $accountHash
     * @return bool
     */
    public function setAccountHash($accountHash)
    {
        $this->accountHash = filter_var(trim(rtrim($accountHash)), FILTER_SANITIZE_STRING);
        return update_option($this->_pluginScreenName . 'AccountHash', $accountHash);
    }

    /**
     * Set the environment code
     * @param $envId
     * @return bool
     */
    public function setEnvId($envId)
    {
        $this->envId = filter_var(trim(rtrim($envId)), FILTER_SANITIZE_STRING);
        return update_option($this->_pluginScreenName . 'EnvId', $envId);
    }


    /**
     * New playback shortcode
     * @param $atts
     * @return string
     */
    public function newPlayback($atts)
    {

        global $wpdb;
        $recordingId = filter_var($atts[0], FILTER_SANITIZE_NUMBER_INT);
        $html = '';

        // Check if the user has placed a recording id
        if (empty($recordingId)) {
            return $html = "You must enter a recording id for the playback. Please copy the correct shortcode from the plugin's Recordings section.";
        }

        // Grab the recording
        // phpcs:disable
        $recordingDetails = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}addpipe_records` WHERE `recording_id` = %d LIMIT 1", $recordingId));
        // phpcs:enable
        $recordingDetails = $recordingDetails[0];

        $html .= "
                <style type='text/css'>
                    .addpipe-playback {
                        text-align: center;
                        width: 100%;
                        height: 100%;
                    }
                    .player-playback {
                        border-radius: 5px;
                    }
                </style>
                <div class='addpipe-playback'>
                    <video class='player-playback' poster='{$recordingDetails->recording_img_url}' controls>
                        <source src='{$recordingDetails->recording_url}' type='video/mp4'>
                        <strong>Your browser does not support the video tag. Please use a newer browser (Chrome, Firefox).</strong>
                    </video>
                </div>
        ";

        return $html;

    }

    /**
     * Embed a new recorder - shortcode
     * * checks whether there are given values (as it should be)
     * * fault tolerant: uses the default values if no custom values are sent/set
     * @param $atts
     * @return string
     */
    public function newRecorder($atts)
    {
        global $wpdb;
        $recorderId = filter_var($atts[0], FILTER_SANITIZE_NUMBER_INT);
        // phpcs:disable
        $shortcodeRecord = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}addpipe_shortcodes` WHERE `shortcode_id` = %d LIMIT 1", $recorderId));
        // phpcs:enable

        // Return empty string if no record is available
        if (count($shortcodeRecord) == 0) {
            return "";
        }

        $shortcodeRecord = $shortcodeRecord[0];

        // If the user forgot to enter a recorder id, we'll just use the default settings
        if (empty($recorderId)) {
            $shortcodeRecord = new stdClass();
            $shortcodeRecord->quality = $this->recordingQuality;
            $shortcodeRecord->length = $this->recordingLength;
        }

        $currentUserId = get_current_user_id();

        $html = "
                <style type='text/css'>
                    #accountHashCheck-missing {
                        color: red;
                        font-weight: bold;
                        padding: 20px;
                        margin-top: 20px;
                        margin-bottom: 20px;
                        background-color: white;
                        border-radius: 15px 15px;
                        text-align: center;
                    }
                    .addpipe-recorder {
                        text-align: center;
                        display: flex;
                        justify-content: center;
                        align-items: center;
                        background-color: rgb(246, 246, 246);
                        min-height: {$shortcodeRecord->height}px;
                    }
                </style>
                <piperecorder id='addpipe-recorder-{$shortcodeRecord->shortcode_id}' class='addpipe-recorder'
                    pipe-width='{$shortcodeRecord->width}'
                    pipe-height='{$shortcodeRecord->height}'
                    pipe-qualityurl='avq/{$shortcodeRecord->quality}.xml'
                    pipe-accountHash='{$this->accountHash}'
                    pipe-eid='{$shortcodeRecord->env_code}'
                    pipe-mrt={$shortcodeRecord->length}
                    pipe-avrec=1
                    pipe-asv={$shortcodeRecord->asv}
                    pipe-ao={$shortcodeRecord->ao}
                    pipe-showMenu={$shortcodeRecord->showMenu}
                    pipe-sis={$shortcodeRecord->sis}
                    pipe-dup={$shortcodeRecord->dup}
                    pipe-srec={$shortcodeRecord->srec}
                    pipe-countdowntimer={$shortcodeRecord->countdowntimer}
                    pipe-bgblur={$shortcodeRecord->bgblur}
                    pipe-payload='{$this->accountHash},{$currentUserId},{$this->getThePostId()}'
                >
                    <p>Loading Pipe Recorder</p>
                </piperecorder>
        ";

        return $html;

    }

}
