<?php

// No direct access
if (!defined('ABSPATH')) {
    die('Error #845173685');
}

/*
 * Get all recordings
 *
 * $limit - how many to pull at once
 * $offset - from which index should we start displaying
 */
$recordings = $this->getRecordedRecordings();

?>

<style type="text/css">
    .widefat td,
    .widefat th {
        vertical-align: middle;
    }

    .widefat tr:hover {
        background-color: #F1F1F1;
    }

    .widefat tr,
    .widefat th {
        text-align: center;
    }

    .sync-deleted-recordings {
        padding: 4px;
        color: white;
        background-color: darkslategray;
        border: 0;
        -webkit-border-radius: 3px;
        -moz-border-radius: 3px;
        border-radius: 3px;
        cursor: hand;
        cursor: pointer;
    }

    .delete-recording {
        color: #FF3937 !important;
        border-color: #FF3937 !important;
    }

    .slim-btn {
        padding: 0 5px !important;
        min-height: 20px !important;
        line-height: 1.5 !important;
        font-size: 12px !important;
        width: 100% !important;
        margin-block: 2px !important;
    }

    .flex-items {
        gap: 5px;
        display: flex;
        flex-wrap: wrap;
        align-items: center;
        justify-content: center;
    }
</style>

<div class="wrap">

    <!-- Logo/title -->
    <h3>Manage your recordings</h3>

    <!-- If we have recordings, show the sync deleted option -->
    <!-- <?php echo (!empty($recordings)) ? "<h4>Have you deleted recordings from your addpipe.com account? Don't forget to <button class='sync-deleted-recordings'>Sync Deleted Recordings</button></h4>" : ''; ?> -->

    <!-- Start the listing -->
    <table class="widefat">
        <thead>
            <tr>
                <th>Date inserted</th>
                <th>Recording ID • Environment code</th>
                <th>Preview</th>
                <th>Recording details</th>
                <th>Actions</th>
                <th>Playback embed code</th>
                <th>Delete</th>
            </tr>
        </thead>
        <tbody>

            <!-- Check whether we have recordings -->
            <?php if (!empty($recordings)) { ?>
                <!-- Iterate through all recordings that are ACTIVE -->
                <?php foreach ($recordings as $arr => $obj) { ?>

                    <tr id='<?php echo esc_attr($obj->recording_id); ?>'>
                        <td>
                            <!-- Date -->
                            <?php echo esc_attr((new DateTime($obj->date, new DateTimeZone('UTC')))->setTimezone(wp_timezone())->format('Y-m-d H:i:s')); ?>
                        </td>
                        <td>
                            <!-- Recording ID -->
                            <?php echo esc_attr($obj->recording_id) . ' • ' . esc_attr($obj->environment_code); ?>
                        </td>
                        <td>
                            <!-- Img (snapshot/thumbnail) -->
                            <img src="<?php echo esc_url($obj->recording_img_url); ?>" width="150px" onerror="imageErrorHandler(this, <?php echo $obj->recording_img_url ? 'true' : 'false'; ?>, <?php echo ($obj->recording_img_url === 'audio_file') ? 'true' : 'false'; ?>);" />
                        </td>
                        <td>
                            <!-- Details (stream name, length, size) -->
                            Stream name: <code><?php echo esc_attr($obj->stream_name); ?></code>
                            <br />
                            Original extension: <code><?php echo esc_attr($obj->extension); ?></code>
                            <?php if ($obj->recording_size !== null && $obj->recording_size !== "0.00"): ?>
                                <br />
                                <?php echo esc_attr($this->filesizeForHumans($obj->recording_size)); ?>
                            <?php endif; ?>
                            <?php if ($obj->recording_length !== null && $obj->recording_length !== "0.00"): ?>
                                <?php echo '• ' . esc_attr($obj->recording_length) . ' seconds'; ?>
                            <?php endif; ?>
                            <?php if ($obj->recording_width !== null && $obj->recording_width != 0 && $obj->recording_height !== null): ?>
                                <?php echo '• ' . esc_attr($obj->recording_width) . 'x' . esc_attr($obj->recording_height); ?>
                                <br />
                            <?php elseif ($obj->recording_width !== null && $obj->recording_width == 0): ?>
                                <?php echo '• audio only'; ?>
                                <br />
                            <?php else: ?>
                                <br />
                            <?php endif; ?>
                            By:
                            <!-- User who made the recording -->
                            <?php $user_data = get_userdata($obj->user_id); ?>
                            <?php echo ($user_data) ? esc_attr($user_data->user_login) : '<em>visitor</em>'; ?>
                        </td>
                        <td>
                            <!-- Control buttons: Play, Download MP4, Download RAW; -->
                            <?php if ($obj->recording_url !== null && $obj->recording_url !== ""): ?>
                                <a class="button-secondary playvideo slim-btn" href="<?php echo esc_url($obj->recording_url); ?>" target="_blank" title="Play MP4">Play MP4 </a>
                                <a class="button-secondary slim-btn" href="<?php echo $this->getPluginViewURL("download.php") . '?file=' . urlencode($obj->recording_url); ?>" title="Download MP4">Download MP4</a>
                                <?php endif; ?>
                            <?php if ($obj->recording_raw_url !== null && $obj->recording_raw_url !== ""): ?>
                                <a class="button-secondary slim-btn" href="<?php echo $this->getPluginViewURL("download.php") . '?file=' . urlencode($obj->recording_raw_url); ?>" title="Download RAW">Download <?php echo strtoupper($obj->extension); ?></a>
                            <?php endif; ?>
                        </td>
                        <td>
                            <p>
                                <!-- Shortcode input text -->
                                <?php if ($obj->recording_url !== null && $obj->recording_url !== ""): ?>
                                    <input type="text" readonly value="[pipe_playback <?php echo esc_attr($obj->recording_id); ?>]" />
                                <?php endif; ?>
                            </p>
                        </td>
                        <td>
                            <!-- Delete button -->
                            <button class="button-secondary delete-recording slim-btn" data-id="<?php echo esc_attr($obj->recording_id); ?>">Delete</button>
                        </td>
                    </tr>

                    <?php

                } // End foreach
            
            } else {

                echo '
                       <td colspan="6">
                        <h3>There are no entries in the local database.</h3>
                        <p>If you have recorded anything through the plugin but the recording does not show up here, make sure you`ve correctly set up the webhook in your <a href="https://addpipe.com" target="_blank">addpipe.com</a> account. <a href="https://addpipe.com/docs/plugins/wordpress-plugin/" target="_blank">Learn more</a>.</p>
                       </td>';
            }

            ?> <!-- End if check -->

        </tbody>
        <tfoot>
            <tr>
                <th>Date inserted</th>
                <th>Recording ID • Environment code</th>
                <th>Preview</th>
                <th>Recording details</th>
                <th>Action</th>
                <th>Playback embed code</th>
                <th>Delete</th>
            </tr>
        </tfoot>
    </table>
    <!-- Footer notes -->
    <p>The above table will only show recordings made through the <a href="<?php echo esc_url(admin_url()); ?>admin.php?page=pipe-recorder/load/embed-recorder">plugin's shortcodes</a>.</p>
    <p>Deleting a recording through the table above only deletes the entry from the local WP database. To delete the recording files and entry from the Pipe database you'll have to delete the recording from the <a href="https://dashboard.addpipe.com/recordings" target="_blank">Pipe account dashboard</a>.</p>
    <p>Snapshots, playback links and playback embed codes will work when you use Pipe Platform's complimentary storage and when the respective output is enabled in Pipe Platform's <a href="https://dashboard.addpipe.com/transcoding" target="_blank">Transcoding Engine</a>.</p>
    <p>Depending on the recording's video quality and it's length, it can take from a few seconds up to a couple of minutes for a recording to show up in the above table.</p>
    <p>For more information on the recordings above go to the Pipe account dashboard's <a href="https://dashboard.addpipe.com/recordings" target="_blank">Recordings section</a>.</p>
    <div id="dialog"></div>
</div>
<script type="text/javascript">

    /* Replace the missing recording image */
    function imageErrorHandler(img, whReceived, audioOnly) {
        img.onerror = '';
        // If audio only, display audio only image
        if (audioOnly) {
            img.src = '<?php echo esc_url($this->getPluginViewURL('img/audio_only.png')); ?>';
            return true;
        }
        // If the webhook was received, display 404 (recording was deleted); else display nothing
        img.src = whReceived ? '<?php echo esc_url($this->getPluginViewURL('img/404.png')); ?>' : '';
        return true;
    }

    /* Sync deleted recordings - ajax button */
    jQuery(document).ready(function ($) {

        // Handle button click for deleting a recording
        $('.delete-recording').click(function () {
            var btn = $(this);
            var recordingId = btn.data('id'); // Get the recording ID from the button's data attribute

            if (confirm("Are you sure you want to delete this recording from the local WP database?")) {
                // Disable the button and show a deleting message
                btn.prop('disabled', true).html('Deleting...');

                // Prepare the data to send to the server
                var data = {
                    action: 'delete_recording', // This is the action name that will be handled in the PHP function
                    recording_id: recordingId, // Pass the recording ID
                    _wpnonce: "<?php echo esc_attr(wp_create_nonce('delete_recording_nonce')); ?>" // Nonce for security
                };

                // Send the AJAX request to the server
                $.post(ajaxurl, data, function (response) {
                    if (response.success) {
                        $(`#${recordingId}`).remove(); // Remove the recording element from the page
                    } else {
                        btn.prop('disabled', false).html('Delete');
                        alert('Failed to delete recording: ' + response.data.message);
                    }
                });
            }
        });

        var btnSyncDeleted = $('.sync-deleted-recordings');
        btnSyncDeleted.click(function () {
            $(this).html('Synchronizing...');

            setTimeout(function () {
                data = {
                    action: 'addpipe_ajax_sync_deleted',
                    _wpnonce: "<?php echo esc_attr(wp_create_nonce('addpipe_ajax_sync_deleted')); ?>"
                };
                $.post(ajaxurl, data, function (response) {
                    btnSyncDeleted.html(response);
                });
            }, 650);
        });

        // Open the webhook setup section
        $("#expand-webhook-section").click(() => {
            $("#webhook-section").slideToggle("slow");
        })
    });
</script>